package testnfc;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.event.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.lang.reflect.Method;
import java.util.HashMap;

import utildll.*;

public class TestNFC01 extends JFrame implements Runnable, ActionListener {
	Wrp_felica_nfc_library wnfc = null;
	Wrp_Kernel32 wkrnl = null;
	Wrp_User32 wuser = null;
	Wrp_JInvoker wjinv = null;
	long hWnd = 0;
	long WndProc = 0;
	boolean blInitialized = false, blOpened = false, blStarted = false, blAccessed = false;
	Func_HookProc fhp;
	long pNewProc = 0, pOldProc = 0;
	int card_find_message = 0, card_enable_message = 0;
	LockParam lockprm = new LockParam();
	Param prmin = new Param();
	HashMap<Integer, String> errMap = new HashMap<Integer, String>();
	
	final int BORDER_WIDTH = 4;
	final int TITLE_HEIGHT = 30;
	final int WIDTH = 640;
	final int HEIGHT = 480;
	final int STARTX = 20;
	final int STARTY = 20;
	final int ROW_HEIGHT = 80;
	final int LABEL_HEIGHT = 40;
	final int LABEL_WIDTH = 600;
	final int BUTTON_HEIGHT = 40;
	final int BUTTON_WIDTH = 120;
	final int STATUS_HEIGHT = 40;
	final int STATUS_WIDTH = WIDTH;
	
	JLabel[] lblInfo = new JLabel[4];
	JLabel lblSts;
	JButton btnExit;
	boolean blExit = false;

	public static void main(String[] args) {
		TestNFC01 nfc = new TestNFC01();
	}

	TestNFC01() {
		this.setTitle("NFC_TEST1");
		this.setSize(WIDTH + BORDER_WIDTH * 2, HEIGHT + TITLE_HEIGHT + BORDER_WIDTH);
		this.setLocationRelativeTo(null);
		this.setLayout(null);
		this.setResizable(false);
		
		for (int i = 0; i < lblInfo.length; i++) {
			lblInfo[i] = new JLabel();
			lblInfo[i].setBounds(STARTX, STARTY + ROW_HEIGHT * i, LABEL_WIDTH, LABEL_HEIGHT);
			lblInfo[i].setFont(new Font("MS Gothic", Font.BOLD, 16));
			getContentPane().add(lblInfo[i]);
		}
		
		lblInfo[0].setText("J[hށF");
		lblInfo[1].setText("cF");
		lblInfo[2].setText("J[hIDF");
		lblInfo[3].setText("IDmF");
		
		btnExit = new JButton("  Exit  ");
		btnExit.addActionListener(this);
		btnExit.setBounds(WIDTH - STARTX - BUTTON_WIDTH, STARTY + ROW_HEIGHT * 4, BUTTON_WIDTH, BUTTON_HEIGHT);
		getContentPane().add(btnExit);
		
		lblSts = new JLabel("Edy/nanacoJ[hJ[h[_[ɂĂB");
		lblSts.setBounds(0, HEIGHT - STATUS_HEIGHT, STATUS_WIDTH, STATUS_HEIGHT);
		lblSts.setFont(new Font("MS Gothic", Font.BOLD, 12));
		lblSts.setForeground(Color.blue);
		lblSts.setBorder(new EtchedBorder(EtchedBorder.RAISED));
		getContentPane().add(lblSts);
		
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent ev) {
				System.exit(0);
			}
		});

		this.setVisible(true);
		
		Thread thread = new Thread(this);
		thread.start();
	}

	public void actionPerformed(ActionEvent e) {
		JButton btn = (JButton)e.getSource();
		if (btn == btnExit) {
			if (hWnd != 0) { 
				try {
					wuser.SendMessage(hWnd, H_WinUser.WM_APP, 0, 0);
				}
				catch (Exception_Win64 ex) {
					System.out.println("***** Exception_Win64: " + ex.getMessage());
				}
			}
		}
	}
	
	public void run() {
		try {
			initialize();
			readCard();
		}
		catch (Exception_NFC e) {
			System.out.println("***** Exception_NFC: " + e.getMessage());
			getNFCLastError();
		}
		catch (Exception_Win64 e) {
			System.out.println("***** Exception_Win64: " + e.getMessage());
		}
		finally {
			System.out.println("finally start ");
			try {
				terminate();
			}
			catch (Exception_NFC e) {
				System.out.println("***** Exception_NFC: " + e.getMessage());
				getNFCLastError();
			}
			catch (Exception_Win64 e) {
				System.out.println("***** Exception_Win64: " + e.getMessage());
			}
			System.out.println("finally end ");
		}
		System.exit(0);
	}
	
	void initialize() throws Exception_NFC, Exception_Win64 {
		wnfc = new Wrp_felica_nfc_library();
		wkrnl = new Wrp_Kernel32();
		wuser = new Wrp_User32();
		wjinv = new Wrp_JInvoker();
		initErrMap();
		
		wnfc.felicalib_nfc_initialize();
		blInitialized = true;
		wnfc.felicalib_nfc_open("USB0");
		blOpened = true;
		
		hWnd = getHWnd(this);
		System.out.println("getHWnd hWnd: " + Long.toHexString(hWnd));

		final String msg_str_of_find = "find";
		final String msg_str_of_enable = "enable";
		wnfc.felicalib_nfc_set_poll_callback_parameters(hWnd, msg_str_of_find, msg_str_of_enable);
		
		card_find_message = wuser.RegisterWindowMessage(msg_str_of_find);
		card_enable_message = wuser.RegisterWindowMessage(msg_str_of_enable);
		System.out.println("RegisterWindowMessage card_find_message: " + Integer.toHexString(card_find_message)
				+ " card_enable_message: " + Integer.toHexString(card_enable_message));
		
		fhp = new Func_HookProc("WindowProc", Func.RTNTYPE_INT, new Arg_WindowProc(), this);
		pNewProc = wjinv.allocFunc(fhp);
		System.out.println("allocFunc pNewProc: " + Long.toHexString(pNewProc));

		pOldProc = wuser.SetWindowLongPtr(hWnd, H_WinUser.GWLP_WNDPROC, pNewProc);
		
		final int target_device = H_felica_nfc_library.DEVICE_TYPE_NFC_18092_212K
				| H_felica_nfc_library.DEVICE_TYPE_NFC_18092_424K;
		
		wnfc.felicalib_nfc_start_poll_mode(target_device);
		blStarted = true;
		
	}

	void readCard() throws Exception_NFC, Exception_Win64 {
		Param prmout = new Param();
		
		while (!lockprm.getParam(prmout)) {
			if (prmout.enablef) {
				readNFC();
			}
		}
	}

	void terminate() throws Exception_NFC, Exception_Win64 {
		if (blAccessed) {
			wnfc.felicalib_nfc_stop_dev_access(
				H_felica_nfc_library.RE_NOTIFICATION_SAME_DEVICE);
			blAccessed = false;
		}
		
		if (blStarted) {
			wnfc.felicalib_nfc_stop_poll_mode();
			blStarted = false;
		}
		
		if (blOpened) {
			wnfc.felicalib_nfc_close();
			blOpened = false;
		}
		
		if (blInitialized) {
			wnfc.felicalib_nfc_uninitialize();
			blInitialized = false;
		}
		
		if (pOldProc != 0) {
			wuser.SetWindowLongPtr(hWnd, H_WinUser.GWLP_WNDPROC, pOldProc);
			pOldProc = 0;
		}
		
		if (pNewProc != 0) {
			wjinv.freeObject(pNewProc);
			pNewProc = 0;
		}
		
	}

	int WindowProc(Arg_WindowProc awp) {
		int umsg = awp.getuMsg();
		
		if (umsg == card_find_message) {
			Struct_DEVICE_INFO devinf = new Struct_DEVICE_INFO();
			
			long pdevinf = awp.getlParam();
			System.out.println("lParam: " + Long.toHexString(pdevinf));
			
			if (pdevinf != 0) {
				wjinv.movePtrToStruct(devinf, pdevinf);
				int target_device = devinf.target_device;
				System.out.println("devinf.target_device: " + Integer.toHexString(target_device));
				
				if (target_device == H_felica_nfc_library.DEVICE_TYPE_NFC_18092_212K
						|| target_device == H_felica_nfc_library.DEVICE_TYPE_NFC_18092_424K) {
					Struct_DEVICE_DATA_NFC_18092_212_424K devdata = new Struct_DEVICE_DATA_NFC_18092_212_424K();
					wjinv.movePtrToStruct(devdata, pdevinf + wjinv.sizeof(devinf));
					int target_number = devdata.target_number;
					System.out.println("devdata.target_number: " + Integer.toHexString(target_number));
					
					try {
						wnfc.felicalib_nfc_start_dev_access(target_number);
						blAccessed = true;
						System.out.println("start_dev_access");
					} catch (Exception_NFC e) {
						System.out.println("***** Exception_NFC: " + e.getMessage());
					}
				}
			}
			
		}
		else if (umsg == card_enable_message) {
			if (blAccessed) {
				prmin.enablef = true;
				lockprm.setParam(prmin);
			}
		}
		
		int lrslt = 0;
		try {
			lrslt = wuser.CallWindowProc(pOldProc,
					awp.gethwnd(), awp.getuMsg(), awp.getwParam(), awp.getlParam());
		} catch (Exception_Win64 e) {
			System.out.println("***** Exception_Win64: " + e.getMessage());
		}
		
		if (umsg == H_WinUser.WM_APP) {
			if (pOldProc != 0) {
				try {
					wuser.SetWindowLongPtr(hWnd, H_WinUser.GWLP_WNDPROC, pOldProc);
					pOldProc = 0;
				} catch (Exception_Win64 e) {
					System.out.println("***** Exception_Win64: " + e.getMessage());
				}
			}
			prmin.exitf = true;
			lockprm.setParam(prmin);
		}
		
		return lrslt;
	}

	void readNFC() throws Exception_NFC, Exception_Win64 {
		byte[] bytCmd = new byte[25];
		byte[] bytRsp = new byte[13 + 16 * 2];
		byte[] bytIDm = new byte[8];
		short shtCmdLen = 0;
		short[] shtRspLen = {0};
		boolean blRtn;
		int zangaku;
		
	    //Polling
		for (int i = 0; i < bytCmd.length; i++) bytCmd[i] = 0;
		for (int i = 0; i < bytRsp.length; i++) bytRsp[i] = 0;
		
	    bytCmd[0] = 6; //
	    bytCmd[1] = 0; //R}hR[h
	    bytCmd[2] = (byte)0xFE; //VXeR[h 0x00FE
	    bytCmd[3] = 0x0;
	    bytCmd[4] = 0; //NGXgR[h
	    bytCmd[5] = 0; //^CXbg
	    shtCmdLen = 6;
	    shtRspLen[0] = 18;
	    
	    blRtn = wnfc.felicalib_nfc_thru(bytCmd, shtCmdLen, bytRsp, shtRspLen);
	    
		System.out.println("felicalib_nfc_thru(Polling) blRtn: " + Boolean.toString(blRtn)
				+ " shtRspLen: " + Short.toString(shtRspLen[0])
				+ " bytRsp: " + bytAryToStr(bytRsp, shtRspLen[0], " "));
	    
		
		if (blRtn) {
			
			//IDmۑ
			for (int i = 0; i < bytIDm.length; i++) bytIDm[i] = bytRsp[2 + i];
			
			
		    //Request Service(Edyp:0x170F/nanacop:0x564F)
			for (int i = 0; i < bytCmd.length; i++) bytCmd[i] = 0;
			for (int i = 0; i < bytRsp.length; i++) bytRsp[i] = 0;
			
		    bytCmd[0] = 15; //
		    bytCmd[1] = 2;  //R}hR[h
		    for (int i = 0; i < bytIDm.length; i++) bytCmd[2 + i] = bytIDm[i];
		    bytCmd[10] = 2; //GAET[rX
		    bytCmd[11] = 0xF; //GAET[rXR[h Edyp:0x170F
		    bytCmd[12] = 0x17;
		    bytCmd[13] = 0x4F; //GAET[rXR[h nanacop:0x564F
		    bytCmd[14] = 0x56;
		    shtCmdLen = 15;
		    shtRspLen[0] = 15;
		    
		    blRtn = wnfc.felicalib_nfc_thru(bytCmd, shtCmdLen, bytRsp, shtRspLen);
		    
			System.out.println("felicalib_nfc_thru(Request Service) blRtn: " + Boolean.toString(blRtn)
					+ " shtRspLen: " + Short.toString(shtRspLen[0])
					+ " bytRsp: " + bytAryToStr(bytRsp, shtRspLen[0], " "));
		    
		    
			if (blRtn) {
				
				if (bytRsp[11] != (byte)0xff || bytRsp[12] != (byte)0xff) {
					System.out.println("card kind: Edy");
					
			        //Read Without Encryption(Edyp:0x170F)
					for (int i = 0; i < bytCmd.length; i++) bytCmd[i] = 0;
					for (int i = 0; i < bytRsp.length; i++) bytRsp[i] = 0;
					
			        bytCmd[0] = 16; //
			        bytCmd[1] = 6;  //R}hR[h
			        for (int i = 0; i < bytIDm.length; i++) bytCmd[2 + i] = bytIDm[i];
			        bytCmd[10] = 1; //GAET[rX
			        bytCmd[11] = 0xF; //GAET[rXR[h Edyp:0x170F
			        bytCmd[12] = 0x17;
			        bytCmd[13] = 1; //ubN
			        bytCmd[14] = (byte)0x80; //ubNԍ
			        bytCmd[15] = 0x0;
			        shtCmdLen = 16;
				    shtRspLen[0] = 13 + 16;
				    
				    blRtn = wnfc.felicalib_nfc_thru(bytCmd, shtCmdLen, bytRsp, shtRspLen);
				    
					System.out.println("felicalib_nfc_thru(Read Without Encryption) blRtn: " + Boolean.toString(blRtn)
							+ " shtRspLen: " + Short.toString(shtRspLen[0])
							+ " bytRsp: " + bytAryToStr(bytRsp, shtRspLen[0], " "));
					
					if (blRtn) {
						//c
						zangaku = ((bytRsp[13 + 12] & 0xff) << 24)
								+ ((bytRsp[13 + 13] & 0xff) << 16)
								+ ((bytRsp[13 + 14] & 0xff) << 8)
								+ (bytRsp[13 + 15] & 0xff);
						
				        //Read Without Encryption(Edy:0x110B)
						for (int i = 0; i < bytCmd.length; i++) bytCmd[i] = 0;
						for (int i = 0; i < bytRsp.length; i++) bytRsp[i] = 0;
						
				        bytCmd[0] = 18; //
				        bytCmd[1] = 6;  //R}hR[h
						for (int i = 0; i < bytIDm.length; i++) bytCmd[2 + i] = bytIDm[i];
				        bytCmd[10] = 1; //GAET[rX
				        bytCmd[11] = 0xB; //GAET[rXR[h Edy:0x110B
				        bytCmd[12] = 0x11;
				        bytCmd[13] = 2; //ubN
				        bytCmd[14] = (byte)0x80; //ubNԍ
				        bytCmd[15] = 0x0;
				        bytCmd[16] = (byte)0x80; //ubNԍ
				        bytCmd[17] = 0x1;

				        shtCmdLen = 18;
					    shtRspLen[0] = 13 + 16 * 2;
					    
					    blRtn = wnfc.felicalib_nfc_thru(bytCmd, shtCmdLen, bytRsp, shtRspLen);
					    
						System.out.println("felicalib_nfc_thru(Read Without Encryption) blRtn: " + Boolean.toString(blRtn)
								+ " shtRspLen: " + Short.toString(shtRspLen[0])
								+ " bytRsp: " + bytAryToStr(bytRsp, shtRspLen[0], " "));
						
						if (blRtn) {
							//Edy ID
							StringBuilder strID = new StringBuilder();
							for (int i = 0; i < 8; i++) {
								strID.append(Integer.toHexString((bytRsp[13 + 2 + i] & 0xF0) >> 4));
								strID.append(Integer.toHexString(bytRsp[13 + 2 + i] & 0x0F));
								if ((i % 2) == 1 && i <= 5) strID.append("-");
							}
							
							//\
							lblInfo[0].setText("J[hށF@Edy");
							lblInfo[1].setText(String.format("cF@%1$,3d ~", zangaku));
							lblInfo[2].setText("Edy IDF@" + strID.toString());
							lblInfo[3].setText("IDmF@" + bytAryToStr(bytIDm, bytIDm.length, ""));
						}
					}
				}
				else if (bytRsp[13] != (byte)0xff || bytRsp[14] != (byte)0xff) {
					System.out.println("card kind: nanaco");
					
			        //Read Without Encryption(nanacop:0x564F)
					for (int i = 0; i < bytCmd.length; i++) bytCmd[i] = 0;
					for (int i = 0; i < bytRsp.length; i++) bytRsp[i] = 0;
					
			        bytCmd[0] = 16; //
			        bytCmd[1] = 6;  //R}hR[h
			        for (int i = 0; i < bytIDm.length; i++) bytCmd[2 + i] = bytIDm[i];
			        bytCmd[10] = 1; //GAET[rX
			        bytCmd[11] = 0x4F; //GAET[rXR[h nanacop:0x564F
			        bytCmd[12] = 0x56;
			        bytCmd[13] = 1; //ubN
			        bytCmd[14] = (byte)0x80; //ubNԍ
			        bytCmd[15] = 0x0;
			        shtCmdLen = 16;
				    shtRspLen[0] = 13 + 16;
				    
				    blRtn = wnfc.felicalib_nfc_thru(bytCmd, shtCmdLen, bytRsp, shtRspLen);
				    
					System.out.println("felicalib_nfc_thru(Read Without Encryption) blRtn: " + Boolean.toString(blRtn)
							+ " shtRspLen: " + Short.toString(shtRspLen[0])
							+ " bytRsp: " + bytAryToStr(bytRsp, shtRspLen[0], " "));
					
					if (blRtn) {
						//c
						zangaku = ((bytRsp[13 + 5] & 0xff) << 24)
								+ ((bytRsp[13 + 6] & 0xff) << 16)
								+ ((bytRsp[13 + 7] & 0xff) << 8)
								+ (bytRsp[13 + 8] & 0xff);
						
						
				        //Read Without Encryption(nanacoԍ:0x558B)
						for (int i = 0; i < bytCmd.length; i++) bytCmd[i] = 0;
						for (int i = 0; i < bytRsp.length; i++) bytRsp[i] = 0;
						
				        bytCmd[0] = 18; //
				        bytCmd[1] = 6;  //R}hR[h
						for (int i = 0; i < bytIDm.length; i++) bytCmd[2 + i] = bytIDm[i];
				        bytCmd[10] = 1; //GAET[rX
				        bytCmd[11] = (byte)0x8B; //GAET[rXR[h nanacoԍ:0x558B
				        bytCmd[12] = 0x55;
				        bytCmd[13] = 2; //ubN
				        bytCmd[14] = (byte)0x80; //ubNԍ
				        bytCmd[15] = 0x0;
				        bytCmd[16] = (byte)0x80; //ubNԍ
				        bytCmd[17] = 0x1;
				        
				        shtCmdLen = 18;
					    shtRspLen[0] = 13 + 16 * 2;
					    
					    blRtn = wnfc.felicalib_nfc_thru(bytCmd, shtCmdLen, bytRsp, shtRspLen);
					    
						System.out.println("felicalib_nfc_thru(Read Without Encryption) blRtn: " + Boolean.toString(blRtn)
								+ " shtRspLen: " + Short.toString(shtRspLen[0])
								+ " bytRsp: " + bytAryToStr(bytRsp, shtRspLen[0], " "));
				        
						if (blRtn) {
							//nanacoԍ
							StringBuilder strID = new StringBuilder();
							for (int i = 0; i < 8; i++) {
								strID.append(Integer.toHexString((bytRsp[13 + 0 + i] & 0xF0) >> 4));
								strID.append(Integer.toHexString(bytRsp[13 + 0 + i] & 0x0F));
								if ((i % 2) == 1 && i <= 5) strID.append("-");
							}
							
							//\
							lblInfo[0].setText("J[hށF@nanaco");
							lblInfo[1].setText(String.format("cF@%1$,3d ~", zangaku));
							lblInfo[2].setText("nanacoԍF@" + strID.toString());
							lblInfo[3].setText("IDmF@" + bytAryToStr(bytIDm, bytIDm.length, ""));
						}
					}
				}
				else {
					blRtn = false;
					System.out.println("card kind: unknown");
				}
			}
		}
	    
		if (blRtn) {
			lblSts.setText("J[h\܂B");
		}
		else {
			lblSts.setText("ǂݎG[܂B@ēxJ[hĂB");
		}
		
		if (blAccessed) {
			wnfc.felicalib_nfc_stop_dev_access(
				H_felica_nfc_library.RE_NOTIFICATION_SAME_DEVICE);
			blAccessed = false;
		}
	}
	
	String bytAryToStr(byte[] bytAry, int len, String sep) {
		
		if (len > bytAry.length) len = bytAry.length;
		
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < len; i++) {
			sb.append(Integer.toHexString((bytAry[i] & 0xf0) >> 4));
			sb.append(Integer.toHexString(bytAry[i] & 0x0f));
			if (sep.length() > 0) sb.append(sep);
		}
		return sb.toString();
	}
	
	class Param {
		boolean enablef;
		boolean exitf;
		
		Param(boolean enablef, boolean exitf) {
			this.enablef = enablef;
			this.exitf = exitf;
		}

		Param() {
			this(false, false);
		}
	}
	
	class LockParam {
		private Param prm = new Param();
		private boolean setf = false;
		
		synchronized void setParam(Param prm) {
			this.prm.enablef = prm.enablef;
			this.prm.exitf = prm.exitf;
			this.setf = true;
			notifyAll();
		}
		
		synchronized boolean getParam(Param prm) {
			while (this.setf == false) {
				try {
					wait();
				}  
				catch (InterruptedException e) {
					System.out.println(e);
				}
			}
			prm.enablef = this.prm.enablef;
			prm.exitf = this.prm.exitf;
			this.setf = false;
			notifyAll();
			return this.prm.exitf;
		}
	}

	class Func_HookProc {
		private String _FuncName;
		private int _ReturnType;
		private Object _FuncArg;
		private Object _Instance;
		
		Func_HookProc(String _FuncName, int _ReturnType, Object _FuncArg, Object _Instance) {
			this._FuncName = _FuncName;
			this._ReturnType = _ReturnType;
			this._FuncArg = _FuncArg;
			this._Instance = _Instance;
		}
	}
	
	/*
	LRESULT CALLBACK WindowProc(
	  HWND hwnd,      // EBhẼnh
	  UINT uMsg,      // bZ[W̎ʎq
	  WPARAM wParam,  // bZ[W̍ŏ̃p[^
	  LPARAM lParam   // bZ[W 2 Ԗڂ̃p[^
	);
	 */
	class Arg_WindowProc {
		private long hwnd;
		private int uMsg;
		private long wParam;
		private long lParam;
		
		long gethwnd() {return hwnd;}
		int getuMsg() {return uMsg;}
		long getwParam() {return wParam;}
		long getlParam() {return lParam;}
	}
	
	long getHWnd(Frame f) {
		try {
			Class<?> clsWComponentPeer = Class.forName("sun.awt.windows.WComponentPeer");
			Method methodGetPeer = Frame.class.getMethod("getPeer");
			Object peer = methodGetPeer.invoke(f);
			if (peer == null) {
				return 0;
			}
			Method methodGetHWnd = clsWComponentPeer.getMethod("getHWnd");
			return (Long) methodGetHWnd.invoke(peer);
			
		} catch (Exception ex) {
			ex.printStackTrace();
			return 0;
		}
	}
	
	int[] getNFCLastError() {
		int[] error_info = {0, 0};
		try {
			wnfc.felicalib_nfc_get_last_error(error_info);
		}
		catch (Exception_NFC e) {
			System.out.println("***** Exception_NFC: " + e.getMessage());
		}
		
		System.out.println("getNFCLastError: " + Integer.toHexString(error_info[0])
				+ ", " + Integer.toHexString(error_info[1]));
		
		if (errMap.containsKey(error_info[0])) {
			if (error_info[0] == H_felica_nfc_library_error.FELICA_NFC_E_DRIVER_ERROR) {
				if (errMap.containsKey(error_info[1])) {
					lblSts.setText(errMap.get(error_info[1]));
					System.out.println("getNFCLastError: " + errMap.get(error_info[1]));
				}
				else {
					lblSts.setText("sȃhCoG[R[h: " + Integer.toHexString(error_info[1]));
					System.out.println("getNFCLastError sȃhCoG[R[h: " + errMap.get(error_info[1]));
				}
			}
			else {
				lblSts.setText(errMap.get(error_info[0]));
				System.out.println("getNFCLastError: " + errMap.get(error_info[0]));
			}
		}
		else {
			lblSts.setText("sȃG[R[h: " + Integer.toHexString(error_info[0]));
			System.out.println("getNFCLastError sȃG[R[h: " + errMap.get(error_info[0]));
		}
		
		return error_info;
	}
	
	void initErrMap() {
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_ARGUMENT						, "ُłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_NOT_IMPLEMENT						, "Ă܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_MODE							, "FeliCaCu쒆łB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_MEMORY_ALLOCATE						, "mۂł܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_NOT_INITIALIZE						, "CuĂ܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_PORT_LIST					, "|[gXgsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_PORT_NAME					, "|[gsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_PLUGINS_DIR					, "vOCfBNgsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_PLUGIN_NAME					, "vOCt@CsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_PLUGIN						, "vOCt@CsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_NO_PLUGIN							, "vOCt@C݂܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_CREATE_THREAD						, "Xbhł܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_NOT_PORT_OPENED						, "|[gI[vĂ܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_NO_CALLBACK_PARAM					, "R[obNp[^ݒ肳Ă܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_NOT_CREATE_MESSAGE					, "R[obNpbZ[Wł܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_TIME_OUT								, "NFCCuŃ^CAEg܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_CALLBACK								, "R[obNĂяoɎs܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_ALREADY_INITIALIZED					, "Cu͂łɏĂ܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_PORT_OPENED							, "łɃ|[gI[vĂ܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DRIVER_STATUS						, "hCoXe[^XsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_TARGET_DEVICE_TYPE			, "ߑ^[QbgfoCX^CvsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_FORBIDDEN_OPERARION					, "֎~Ă鑀łB"); 
		
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_TARGET_DEVICE_NOT_FOUND				, "ߑfoCX܂łB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_TARGET_DEVICE_TIMEOUT				, "ߑfoCXgpJnvԓɂ܂łB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_TARGET_DEVICE_ACCESS					, "^[QbgfoCXANZXł̃G[hCoʒm܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_TARGET_DEVICE						, "^[QbgfoCXANZXł̕sG[hCoʒm܂B"); 
		
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DEVICE_MODE							, "[hȍ~G[hCoŔĂ܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DEVICE_INITIALIZE					, "FeliCa|[gfoCX̏G[łB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DEVICE_TEMPERATURE_ERROR				, "FeliCa|[gfoCXxُԂłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DEVICE_TEMPERATURE_FATAL_ERROR		, "FeliCa|[gfoCXxُԂɂAgpłȂԂłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DEVICE_READER_WRITER_DISCONNECTED	, "FeliCa|[gfoCXؒf܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DEVICE_ERROR							, "FeliCa|[gfoCXŕsȃG[Ă܂B"); 
		
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_REGISTRY								, "WXgANZXɎs܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_DIR							, "tH_݂ȂsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_INVALID_FILE							, "t@C݂ȂsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_FILE_ACCESS							, "t@CANZXɎs܂B"); 
		
		errMap.put(H_felica_nfc_library_error.FELICA_NFCP_E_INVALID_ARGUMENT					, "vOCւُ̈łB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFCP_E_INITIALIZE_DRIVER					, "hCȍɎs܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFCP_E_MEMORY_ALLOCATE						, "mۂł܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFCP_E_INVALID_PORT_NAME					, "|[gsłB"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFCP_E_NOT_PORT_OPENED						, "|[gI[vĂ܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFCP_E_OPENED								, "łɃ|[gI[vĂ܂B"); 
		errMap.put(H_felica_nfc_library_error.FELICA_NFCP_E_INVALID_RESPONSE					, "hCõX|XsłB"); 
		
		errMap.put(H_felica_nfc_library_error.FELICA_NFC_E_DRIVER_ERROR							, "hCoŃG[܂B"); 
		
		//driver error
		errMap.put(0x00000001, "[_^C^܂B");
		errMap.put(0x00000002, "[_^C^܂B");
		errMap.put(0x00000003, "[_^C^܂B");
		errMap.put(0x00000005, "[_^C^܂B");
		errMap.put(0x00000015, "[_^C^G[łBĂȂBxُB");
		errMap.put(0x00000016, "[_^C^܂B");
		errMap.put(0x00000017, "RF ʐMG[AJ[hԃG[A[_^C^xُA܂̓J[h̎Mf[^ُłB");
		errMap.put(0x00000020, "foCX̍őI[v𒴂Ă܂B");
		errMap.put(0x00000032, "API ̔sV[PX܂B");
		errMap.put(0x00000079, "ANZX҂^CAEg܂̓R}h^CAEgĂ܂B");
		errMap.put(0x0000007A, "hCoC^tFCXG[łB");
		errMap.put(0x000000AA, "hCov󂯕tȂԂłB");
		errMap.put(0x000000E9, "RF ʐMG[AJ[hԃG[A[_^C^xُA܂̓J[h̎Mf[^ُłB");
		errMap.put(0x0000045D, "USB ʐMG[܂̓[_^C^X|XpPbgsłB");
		errMap.put(0x0000048F, "[_^C^܂B");
		errMap.put(0x000005AA, "VXe\[XsĂ܂B");
		errMap.put(0x000006F8, "hCoւ̗vp[^G[łB");
	}
	
}
