package testnfc;

import utildll.*;

public class Wrp_User32 {
	static final int CHARSET = Arg.ANSI;
	static final String USER32_DLL_NAME = "user32";
	private String[] strFncName = {
			"CreateWindowExA",
			"DestroyWindow",
			"LoadCursorA",
			"RegisterClassExA",
			"DefWindowProcA",
			"RegisterWindowMessageA",
			"GetMessageA",
			"TranslateMessage",
			"DispatchMessageA",
			"PeekMessageA",
			"PostQuitMessage",
			"FindWindowA",
			"SetWindowLongPtrA",
			"CallWindowProcA",
			"SendMessageA"};
	private long[] lngFncAdrs = new long[strFncName.length];
	private final int 
			IDX_CreateWindowEx = 0,
			IDX_DestroyWindow = 1,
			IDX_LoadCursor = 2,
			IDX_RegisterClassEx = 3,
			IDX_DefWindowProc = 4,
			IDX_RegisterWindowMessage = 5,
			IDX_GetMessage = 6,
			IDX_TranslateMessage = 7,
			IDX_DispatchMessage = 8,
			IDX_PeekMessage = 9,
			IDX_PostQuitMessage = 10,
			IDX_FindWindow = 11,
			IDX_SetWindowLongPtr = 12,
			IDX_CallWindowProc = 13,
			IDX_SendMessage = 14;
	private long hMod = 0;
	
	Wrp_User32() {
		
		Arg_GetModuleHandle agmh = new Arg_GetModuleHandle(USER32_DLL_NAME);
		
		hMod = JInvoker.callLongProc("GetModuleHandle", "kernel32", agmh);
		
		System.out.println("GetModuleHandle rtn: " + Long.toHexString(hMod)
				+ " LastError: " + Integer.toString(agmh.getLastError()));
		
		for (int i = 0; i < strFncName.length; i++) {
			Arg_GetProcAddress agpa = new Arg_GetProcAddress(hMod, strFncName[i]);
			
			lngFncAdrs[i] = JInvoker.callLongProc("GetProcAddress", "kernel32", agpa);
			
			System.out.println("GetProcAddress rtn: " + Long.toHexString(lngFncAdrs[i])
					+ " LastError: " + Integer.toString(agpa.getLastError())
					+ " FncName: " + strFncName[i]);
		}
	}

	class Arg_property {
		private int _LastError;
	
		int getLastError() {return _LastError;}
	}

	/*
	HWND CreateWindowEx(
	    DWORD dwExStyle,
	    LPCTSTR lpClassName,
	    LPCTSTR lpWindowName,
	    DWORD dwStyle,
	    int X,
	    int Y,
	    int nWidth,
	    int nHeight,
	    HWND hWndParent,
	    HMENU hMenu,
	    HINSTANCE hInstance,
	    LPVOID lpParam
	);
	 */
	@_CharSet(CHARSET)
	class Arg_CreateWindowEx extends Arg_property {
		private int dwExStyle;
		private @_Const String lpClassName;
		private @_Const String lpWindowName;
		private int dwStyle;
		private int X;
		private int Y;
		private int nWidth;
		private int nHeight;
		private long hWndParent;
		private long hMenu;
		private long hInstance;
		private long lpParam;
		
		Arg_CreateWindowEx(int dwExStyle, String lpClassName, String lpWindowName, int dwStyle, int X, int Y,
				int nWidth, int nHeight, long hWndParent, long hMenu, long hInstance, long lpParam) {
			this.dwExStyle = dwExStyle;
			this.lpClassName = lpClassName;
			this.lpWindowName = lpWindowName;
			this.dwStyle = dwStyle;
			this.X = X;
			this.Y = Y;
			this.nWidth = nWidth;
			this.nHeight = nHeight;
			this.hWndParent = hWndParent;
			this.hMenu = hMenu;
			this.hInstance = hInstance;
			this.lpParam = lpParam;
		}
	}
	
	long CreateWindowEx(int dwExStyle, String lpClassName, String lpWindowName, int dwStyle, int X, int Y,
			int nWidth, int nHeight, long hWndParent, long hMenu, long hInstance, long lpParam)
			throws Exception_Win64 {
		Arg_CreateWindowEx acwe = new Arg_CreateWindowEx(dwExStyle, lpClassName, lpWindowName,
				dwStyle, X, Y, nWidth, nHeight, hWndParent, hMenu, hInstance, lpParam);

		long lngRtn = JInvoker.callLongProcAddress(lngFncAdrs[IDX_CreateWindowEx], acwe);
		
		System.out.println("CreateWindowEx rtn: " + Long.toHexString(lngRtn)
				+ " LastError: " + Integer.toString(acwe.getLastError()));
		
		if (acwe.getLastError() != 0) throw new Exception_Win64("CreateWindowEx");
		if (lngRtn == 0) throw new Exception_Win64("CreateWindowEx");
		
		return lngRtn;
	}
	
	/*
	BOOL DestroyWindow(
	    HWND hWnd
	);
	 */
	class Arg_DestroyWindow extends Arg_property {
		private long hWnd;
		
		Arg_DestroyWindow(long hWnd) {
			this.hWnd = hWnd;
		}
	}
	
	boolean DestroyWindow(long hWnd) throws Exception_Win64 {
		Arg_DestroyWindow adw = new Arg_DestroyWindow(hWnd);

		boolean blRtn = JInvoker.callBooleanProcAddress(lngFncAdrs[IDX_DestroyWindow], adw);
		
		System.out.println("DestroyWindow rtn: " + Boolean.toString(blRtn)
				+ " LastError: " + Integer.toString(adw.getLastError()));
		
		if (adw.getLastError() != 0) throw new Exception_Win64("DestroyWindow");
		if (blRtn == false) throw new Exception_Win64("DestroyWindow");
		
		return blRtn;
	}
	
	/*
	HCURSOR LoadCursor(
	    HINSTANCE hInstance,
	    LPCTSTR lpCursorName
	);
	 */
	@_CharSet(CHARSET)
	class Arg_LoadCursor extends Arg_property {
		private long hInstance;
		private long lpCursorName;
		
		Arg_LoadCursor(long hInstance, long lpCursorName) {
			this.hInstance = hInstance;
			this.lpCursorName = lpCursorName;
		}
	}
	
	long LoadCursor(long hInstance, long lpCursorName) throws Exception_Win64 {
		Arg_LoadCursor alc = new Arg_LoadCursor(hInstance, lpCursorName);

		long lngRtn = JInvoker.callLongProcAddress(lngFncAdrs[IDX_LoadCursor], alc);
		
		System.out.println("LoadCursor rtn: " + Long.toHexString(lngRtn)
				+ " LastError: " + Integer.toString(alc.getLastError()));
		
		if (alc.getLastError() != 0) throw new Exception_Win64("LoadCursor");
		if (lngRtn == 0) throw new Exception_Win64("LoadCursor");
		
		return lngRtn;
	}
	
	/*
	ATOM RegisterClassEx(
	    CONST WNDCLASSEX *lpwcx
	);
	 */
	@_CharSet(CHARSET)
	class Arg_RegisterClassEx extends Arg_property {
		private @_Const Struct_WNDCLASSEX lpwcx;
		
		Arg_RegisterClassEx(Struct_WNDCLASSEX lpwcx) {
			this.lpwcx = lpwcx;
		}
	}
	
	short RegisterClassEx(Struct_WNDCLASSEX lpwcx) throws Exception_Win64 {
		Arg_RegisterClassEx arce = new Arg_RegisterClassEx(lpwcx);

		short shtRtn = JInvoker.callShortProcAddress(lngFncAdrs[IDX_RegisterClassEx], arce);
		
		System.out.println("RegisterClassEx rtn: " + Integer.toHexString(shtRtn)
				+ " LastError: " + Integer.toString(arce.getLastError()));
		
		if (arce.getLastError() != 0) throw new Exception_Win64("RegisterClassEx");
		if (shtRtn == 0) throw new Exception_Win64("RegisterClassEx");
		
		return shtRtn;
	}
	
	/*
	LRESULT DefWindowProc(
	    HWND hWnd,
	    UINT Msg,
	    WPARAM wParam,
	    LPARAM lParam
	);
	 */
	@_CharSet(CHARSET)
	class Arg_DefWindowProc extends Arg_property {
		private long hWnd;
		private int Msg;
		private long wParam;
		private long lParam;
		
		Arg_DefWindowProc(long hWnd, int Msg, long wParam, long lParam) {
			this.hWnd = hWnd;
			this.Msg = Msg;
			this.wParam = wParam;
			this.lParam = lParam;
		}
	}
	
	int DefWindowProc(long hWnd, int Msg, long wParam, long lParam) throws Exception_Win64 {
		Arg_DefWindowProc adwp = new Arg_DefWindowProc(hWnd, Msg, wParam, lParam);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_DefWindowProc], adwp);
		
		System.out.println("DefWindowProc rtn: " + Integer.toString(intRtn)
				+ " LastError: " + Integer.toString(adwp.getLastError()));
		
		if (adwp.getLastError() < 0) throw new Exception_Win64("DefWindowProc");
		
		return intRtn;
	}
	
	/*
	UINT RegisterWindowMessage(
	    LPCTSTR lpString
	);
	 */
	@_CharSet(CHARSET)
	class Arg_RegisterWindowMessage extends Arg_property {
		private @_Const String lpString;
		
		Arg_RegisterWindowMessage(String lpString) {
			this.lpString = lpString;
		}
	}
	
	int RegisterWindowMessage(String lpString) throws Exception_Win64 {
		Arg_RegisterWindowMessage arwm = new Arg_RegisterWindowMessage(lpString);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_RegisterWindowMessage], arwm);
		
		System.out.println("RegisterWindowMessage rtn: " + Integer.toHexString(intRtn)
				+ " LastError: " + Integer.toString(arwm.getLastError()));
		
		if (arwm.getLastError() != 0) throw new Exception_Win64("RegisterWindowMessage");
		if (intRtn == 0) throw new Exception_Win64("RegisterWindowMessage");
		
		return intRtn;
	}

	/*
	BOOL GetMessage(
	    LPMSG lpMsg,
	    HWND hWnd,
	    UINT wMsgFilterMin,
	    UINT wMsgFilterMax
	);
	 */
	@_CharSet(CHARSET)
	class Arg_GetMessage extends Arg_property {
		private Struct_MSG lpMsg;
		private long hWnd;
		private int wMsgFilterMin;
		private int wMsgFilterMax;
		
		Arg_GetMessage(Struct_MSG lpMsg, long hWnd, int wMsgFilterMin, int wMsgFilterMax) {
			this.lpMsg = lpMsg;
			this.hWnd = hWnd;
			this.wMsgFilterMin = wMsgFilterMin;
			this.wMsgFilterMax = wMsgFilterMax;
		}
	}
	
	int GetMessage(Struct_MSG lpMsg, long hWnd, int wMsgFilterMin, int wMsgFilterMax) throws Exception_Win64 {
		Arg_GetMessage agm = new Arg_GetMessage(lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_GetMessage], agm);
		
		System.out.println("GetMessage rtn: " + Integer.toString(intRtn)
				+ " LastError: " + Integer.toString(agm.getLastError()));
		
		if (agm.getLastError() != 0) throw new Exception_Win64("GetMessage");
		if (intRtn == -1) throw new Exception_Win64("GetMessage");
		
		return intRtn;
	}
	
	/*
	BOOL TranslateMessage(
	    CONST MSG *lpMsg
	);
	 */
	@_CharSet(CHARSET)
	class Arg_TranslateMessage extends Arg_property {
		private @_Const Struct_MSG lpMsg;
		
		Arg_TranslateMessage(Struct_MSG lpMsg) {
			this.lpMsg = lpMsg;
		}
	}
	
	int TranslateMessage(Struct_MSG lpMsg) throws Exception_Win64 {
		Arg_TranslateMessage atm = new Arg_TranslateMessage(lpMsg);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_TranslateMessage], atm);
		
		System.out.println("TranslateMessage rtn: " + Integer.toString(intRtn)
				+ " LastError: " + Integer.toString(atm.getLastError()));
		
		if (atm.getLastError() < 0) throw new Exception_Win64("TranslateMessage");
		
		return intRtn;
	}
	
	/*
	LRESULT DispatchMessage(
	    CONST MSG *lpMsg
	);
	 */
	@_CharSet(CHARSET)
	class Arg_DispatchMessage extends Arg_property {
		private @_Const Struct_MSG lpMsg;
		
		Arg_DispatchMessage(Struct_MSG lpMsg) {
			this.lpMsg = lpMsg;
		}
	}
	
	int DispatchMessage(Struct_MSG lpMsg) throws Exception_Win64 {
		Arg_DispatchMessage adm = new Arg_DispatchMessage(lpMsg);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_DispatchMessage], adm);
		
		System.out.println("DispatchMessage rtn: " + Integer.toString(intRtn)
				+ " LastError: " + Integer.toString(adm.getLastError()));
		
		if (adm.getLastError() < 0) throw new Exception_Win64("DispatchMessage");
		
		return intRtn;
	}
	
	/*
	BOOL PeekMessage(
	    LPMSG lpMsg,
	    HWND hWnd,
	    UINT wMsgFilterMin,
	    UINT wMsgFilterMax,
	    UINT wRemoveMsg
	);
	 */
	@_CharSet(CHARSET)
	class Arg_PeekMessage extends Arg_property {
		private Struct_MSG lpMsg;
		private long hWnd;
		private int wMsgFilterMin;
		private int wMsgFilterMax;
		private int wRemoveMsg;
		
		Arg_PeekMessage(Struct_MSG lpMsg, long hWnd, int wMsgFilterMin, int wMsgFilterMax, int wRemoveMsg) {
			this.lpMsg = lpMsg;
			this.hWnd = hWnd;
			this.wMsgFilterMin = wMsgFilterMin;
			this.wMsgFilterMax = wMsgFilterMax;
			this.wRemoveMsg = wRemoveMsg;
		}
	}
	
	int PeekMessage(Struct_MSG lpMsg, long hWnd, int wMsgFilterMin, int wMsgFilterMax, int wRemoveMsg)
			throws Exception_Win64 {
		Arg_PeekMessage apm = new Arg_PeekMessage(lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax, wRemoveMsg);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_PeekMessage], apm);
		
		System.out.println("PeekMessage rtn: " + Integer.toString(intRtn)
				+ " LastError: " + Integer.toString(apm.getLastError()));
		
		if (apm.getLastError() < 0) throw new Exception_Win64("PeekMessage");
		
		return intRtn;
	}
	
	/*
	VOID PostQuitMessage(
	    int nExitCode
	);
	 */
	class Arg_PostQuitMessage extends Arg_property {
		private int nExitCode;
		
		Arg_PostQuitMessage(int nExitCode) {
			this.nExitCode = nExitCode;
		}
	}
	
	void PostQuitMessage(int nExitCode) throws Exception_Win64 {
		Arg_PostQuitMessage apqm = new Arg_PostQuitMessage(nExitCode);

		JInvoker.callVoidProcAddress(lngFncAdrs[IDX_PostQuitMessage], apqm);
		
		System.out.println("PostQuitMessage LastError: " + Integer.toString(apqm.getLastError()));
		
		if (apqm.getLastError() < 0) throw new Exception_Win64("PostQuitMessage");
		
		return;
	}
	
	/*
	HWND FindWindow(
	  LPCTSTR lpClassName,  // NX
	  LPCTSTR lpWindowName  // EBhE
	);
	*/
	@_CharSet(CHARSET)
	class Arg_FindWindow extends Arg_property {
		private @_Const String lpClassName;
		private @_Const String lpWindowName;
		
		Arg_FindWindow(String lpClassName, String lpWindowName) {
			this.lpClassName = lpClassName;
			this.lpWindowName = lpWindowName;
		}
	}

	long FindWindow(String lpClassName, String lpWindowName) throws Exception_Win64 {
		Arg_FindWindow afw = new Arg_FindWindow(lpClassName, lpWindowName);

		long lngRtn = JInvoker.callLongProcAddress(lngFncAdrs[IDX_FindWindow], afw);
		
		System.out.println("FindWindow rtn: " + Long.toHexString(lngRtn)
				+ " LastError: " + Integer.toString(afw.getLastError()));
		
		if (afw.getLastError() != 0) throw new Exception_Win64("FindWindow");
		if (lngRtn == 0) throw new Exception_Win64("FindWindow");
		
		return lngRtn;
	}

	/*
	LONG_PTR SetWindowLongPtr(
	  HWND hWnd,       // EBhẼnh
	  int nIndex,      // ݒ肷l̃ItZbg
	  LONG_PTR dwNewLong   // Vl
	);
	 */
	class Arg_SetWindowLongPtr extends Arg_property {
		private long hWnd;
		private int nIndex;
		private long dwNewLong;
		
		Arg_SetWindowLongPtr(long hWnd, int nIndex, long dwNewLong) {
			this.hWnd = hWnd;
			this.nIndex = nIndex;
			this.dwNewLong = dwNewLong;
		}
	}

	long SetWindowLongPtr(long hWnd, int nIndex, long dwNewLong) throws Exception_Win64 {
		Arg_SetWindowLongPtr aswlp = new Arg_SetWindowLongPtr(hWnd, nIndex, dwNewLong);

		long lngRtn = JInvoker.callLongProcAddress(lngFncAdrs[IDX_SetWindowLongPtr], aswlp);
		
		System.out.println("SetWindowLongPtr rtn: " + Long.toHexString(lngRtn)
				+ " LastError: " + Integer.toString(aswlp.getLastError()));
		
		if (aswlp.getLastError() != 0) throw new Exception_Win64("SetWindowLongPtr");
		if (lngRtn == 0) throw new Exception_Win64("SetWindowLongPtr");
		
		return lngRtn;
	}

	/*
	LRESULT CallWindowProc(
	  WNDPROC lpPrevWndFunc,  // ̃EBhEvV[W
	  HWND hWnd,              // EBhẼnh
	  UINT Msg,               // bZ[W
	  WPARAM wParam,          // bZ[W̍ŏ̃p[^
	  LPARAM lParam           // bZ[W 2 Ԗڂ̃p[^
	);
	 */
	class Arg_CallWindowProc extends Arg_property {
		private long lpPrevWndFunc;
		private long hwnd;
		private int Msg;
		private long wParam;
		private long lParam;
		
		Arg_CallWindowProc(long lpPrevWndFunc, long hwnd, int Msg, long wParam, long lParam) {
			this.lpPrevWndFunc = lpPrevWndFunc;
			this.hwnd = hwnd;
			this.Msg = Msg;
			this.wParam = wParam;
			this.lParam = lParam;
		}
	}

	int CallWindowProc(long lpPrevWndFunc, long hwnd, int Msg, long wParam, long lParam) throws Exception_Win64 {
		Arg_CallWindowProc acwp = new Arg_CallWindowProc(lpPrevWndFunc, hwnd, Msg, wParam, lParam);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_CallWindowProc], acwp);
		/*
		System.out.println("CallWindowProc rtn: " + Integer.toString(intRtn)
				+ " LastError: " + Integer.toString(acwp.getLastError()));
		*/
		if (acwp.getLastError() < 0) throw new Exception_Win64("CallWindowProc");
		
		return intRtn;
	}

	/*
	LRESULT SendMessage(
	    HWND hWnd,
	    UINT Msg,
	    WPARAM wParam,
	    LPARAM lParam
	);
	 */
	@_CharSet(CHARSET)
	class Arg_SendMessage extends Arg_property {
		private long hWnd;
		private int Msg;
		private long wParam;
		private long lParam;
		
		Arg_SendMessage(long hWnd, int Msg, long wParam, long lParam) {
			this.hWnd = hWnd;
			this.Msg = Msg;
			this.wParam = wParam;
			this.lParam = lParam;
		}
	}
	
	int SendMessage(long hWnd, int Msg, long wParam, long lParam) throws Exception_Win64 {
		Arg_SendMessage asm = new Arg_SendMessage(hWnd, Msg, wParam, lParam);

		int intRtn = JInvoker.callIntProcAddress(lngFncAdrs[IDX_SendMessage], asm);
		
		System.out.println("SendMessage rtn: " + Integer.toString(intRtn)
				+ " LastError: " + Integer.toString(asm.getLastError()));
		
		if (asm.getLastError() < 0) throw new Exception_Win64("SendMessage");
		
		return intRtn;
	}
	
	/*
	HMODULE GetModuleHandle(
	  LPCTSTR lpModuleName   // W[
	);
	 */
	@_CharSet(CHARSET)
	class Arg_GetModuleHandle extends Arg_property {
		private @_Const String lpModuleName;

		Arg_GetModuleHandle(String lpModuleName) {
			this.lpModuleName = lpModuleName;
		}
	}

	/*
	HMODULE LoadLibrary(
	  LPCTSTR lpFileName   // W[̃t@C
	);
	 */
	@_CharSet(CHARSET)
	class Arg_LoadLibrary extends Arg_property {
		private @_Const String lpFileName;

		Arg_LoadLibrary(String lpFileName) {
			this.lpFileName = lpFileName;
		}
	}

	/*
	FARPROC GetProcAddress(
	  HMODULE hModule,    // DLL W[̃nh
	  LPCSTR lpProcName   // ֐
	);
	 */
	@_CharSet(Arg.ANSI)
	class Arg_GetProcAddress extends Arg_property {
		private long hModule;
		private @_Const @_Ansi String lpProcName;

		Arg_GetProcAddress(long hModule, String lpProcName) {
			this.hModule = hModule;
			this.lpProcName = lpProcName;
		}
	}
	
	/*
	BOOL FreeLibrary(
		HMODULE hModule   // DLL W[̃nh
	);
	 */
	class Arg_FreeLibrary extends Arg_property {
		private long hModule;

		Arg_FreeLibrary(long hModule) {
			this.hModule = hModule;
		}
	}
	
}
